"use strict";
var __importDefault = (this && this.__importDefault) || function (mod) {
    return (mod && mod.__esModule) ? mod : { "default": mod };
};
Object.defineProperty(exports, "__esModule", { value: true });
exports.ObjectUtils = exports.HexUtils = exports.NumberUtils = exports.AsyncUtils = exports.ErrorUtils = void 0;
// External imports
const assert_1 = __importDefault(require("assert"));
const ethers_1 = require("ethers");
const snarkjs_1 = require("snarkjs");
const crypto_1 = require("crypto");
var ErrorUtils;
(function (ErrorUtils) {
    function ensureError(value) {
        if (value instanceof Error) {
            value.message = '\nError (tornado-sdk): ' + value.message;
            return value;
        }
        let stringified = '[Unable to stringify the thrown value]';
        try {
            stringified = JSON.stringify(value);
        }
        catch { }
        const error = getError(`This value was thrown as is, not through an Error: ${stringified}`);
        return error;
    }
    ErrorUtils.ensureError = ensureError;
    function getError(message) {
        let error = new Error(message);
        error.name = '\nError (tornado-sdk)';
        return error;
    }
    ErrorUtils.getError = getError;
})(ErrorUtils = exports.ErrorUtils || (exports.ErrorUtils = {}));
var AsyncUtils;
(function (AsyncUtils) {
    class PoolPromise extends Promise {
        orderIndex;
    }
    class PromisePooler {
        concurrencyLimit;
        _totalAdded;
        _results;
        _callbacks;
        _errorHandlers;
        _promises;
        constructor(callbacks, errorHandlers, concurrencyLimit) {
            if (callbacks.length == 0)
                throw ErrorUtils.getError('PromisePooler: callbacks are empty.');
            if (concurrencyLimit <= 0)
                throw ErrorUtils.getError("PromisePooler: concurrencyLimit can't be 0 or less.");
            this.concurrencyLimit = concurrencyLimit;
            this._totalAdded = 0;
            this._results = [];
            this._promises = [];
            this._callbacks = callbacks;
            this._errorHandlers = errorHandlers;
        }
        get pending() {
            return this._promises.length;
        }
        get totalAdded() {
            return this._totalAdded;
        }
        async poolMany(...args) {
            for (const arr of args) {
                await this._waitIfFull();
                this._pool(0, this._totalAdded++, ...arr);
            }
        }
        async pool(...args) {
            await this._waitIfFull();
            this._pool(0, this._totalAdded++, ...args);
        }
        async all() {
            return Promise.all(this._results.concat(this._promises)).then((result) => {
                this._results = [];
                return result;
            });
        }
        async race() {
            if (this._results.length != 0) {
                return Promise.resolve(this._results.splice(0, 1)[0]);
            }
            else {
                if (this._promises.length != 0) {
                    // This will actually return the right value
                    // but note that they are stored in results automatically
                    // due to the case of 2 promises not being raced in time
                    // and as such we instead take from results.
                    await Promise.race(this._promises);
                    // This should be synchronous
                    return this.race();
                }
                else
                    return Promise.resolve(null);
            }
        }
        // TODO: Immediately set new callbacks and error handlers
        async reset() {
            let results = await this.all();
            (0, assert_1.default)(this._promises.length === 0, ErrorUtils.getError('PromisePooler.reset: Resetting should have allowed all promises to resolve.'));
            this._results = [];
            this._totalAdded = 0;
            this._callbacks = [];
            this._errorHandlers = [];
            return results;
        }
        _pool(callbackIndex, orderIndex, ...args) {
            let promise = this._callbacks[callbackIndex](...args).then(async (...results) => {
                if (callbackIndex < this._callbacks.length - 1) {
                    // This is synchronous, callbackIndex is never 0
                    return this._pool(callbackIndex + 1, orderIndex, ...results);
                }
                else {
                    let result = results.length == 1 ? results[0] : results;
                    this._promises.splice(this._getPromiseIndex(orderIndex), 1);
                    this._results.push(result);
                    return result;
                }
            }, async (err) => {
                let resolved = this._totalAdded - this.concurrencyLimit;
                resolved = resolved < 0 ? 0 : resolved;
                // Throw inside to abort
                this._errorHandlers[callbackIndex](ErrorUtils.ensureError(err), resolved, callbackIndex, orderIndex, ...args);
                return this._pool(callbackIndex, orderIndex, ...args);
            });
            promise.orderIndex = orderIndex;
            const promiseIndex = this._getPromiseIndex(orderIndex);
            if (promiseIndex < 0) {
                this._promises.push(promise);
            }
            else {
                this._promises[promiseIndex] = promise;
            }
            return promise;
        }
        _getPromiseIndex(orderIndex) {
            return this._promises.findIndex((_promise) => _promise.orderIndex == orderIndex);
        }
        async _waitIfFull() {
            if (this.concurrencyLimit <= this._promises.length) {
                await Promise.race(this._promises);
            }
        }
    }
    AsyncUtils.PromisePooler = PromisePooler;
    function timeout(msTimeout) {
        return new Promise((resolve) => setTimeout(resolve, msTimeout));
    }
    AsyncUtils.timeout = timeout;
    class SimpleMutex {
        _mutexes;
        constructor() {
            this._mutexes = new Map();
        }
        async acquire(parentCallName) {
            let release = () => null;
            const prevMutex = this._mutexes.get(parentCallName);
            this._mutexes.set(parentCallName, new Promise((resolve) => {
                release = resolve;
            }));
            if (prevMutex)
                await prevMutex;
            return release;
        }
    }
    AsyncUtils.SimpleMutex = SimpleMutex;
})(AsyncUtils = exports.AsyncUtils || (exports.AsyncUtils = {}));
var NumberUtils;
(function (NumberUtils) {
    function randomBigInteger(numBytes) {
        // @ts-ignore
        return snarkjs_1.bigInt.leBuff2int((0, crypto_1.randomBytes)(numBytes));
    }
    NumberUtils.randomBigInteger = randomBigInteger;
    function getRandomFromRange(lowerInclusive, upperInclusive, isInteger = 1) {
        isInteger = 0 < isInteger ? 1 : 0;
        return (isInteger ? Math.floor : (x) => x)(Math.random() * (upperInclusive + isInteger - lowerInclusive) + lowerInclusive);
    }
    NumberUtils.getRandomFromRange = getRandomFromRange;
})(NumberUtils = exports.NumberUtils || (exports.NumberUtils = {}));
var HexUtils;
(function (HexUtils) {
    function bufferToHex(buffer, byteLen = 32) {
        return '0x' + buffer.toString('hex').padStart(byteLen * 2, '0');
    }
    HexUtils.bufferToHex = bufferToHex;
    function numberToHex(number, byteLen = 32) {
        return ('0x' +
            ethers_1.BigNumber.from(number)
                .toHexString()
                .slice(2)
                .padStart(2 * byteLen, '0'));
    }
    HexUtils.numberToHex = numberToHex;
    function bigIntToHex(number, byteLen = 32) {
        // @ts-ignore
        return '0x' + number.toString(16).padStart(2 * byteLen, '0');
    }
    HexUtils.bigIntToHex = bigIntToHex;
    function prepareAddress(address, bytelen = 32) {
        return (address.slice(0, 2) == '0x' ? address.slice(2) : address).toLowerCase().padStart(bytelen * 2, '0');
    }
    HexUtils.prepareAddress = prepareAddress;
})(HexUtils = exports.HexUtils || (exports.HexUtils = {}));
var ObjectUtils;
(function (ObjectUtils) {
    function populate(source, target) {
        if (!source) {
            if (!target) {
                throw ErrorUtils.getError('ObjectUtils.populate: source & target object null or undefined');
            }
            else
                return target;
        }
        if (!target)
            throw ErrorUtils.getError('ObjectUtils.populate: target object null or undefined');
        const sourceEntries = Object.entries(source);
        sourceEntries.forEach((sEntry) => {
            const key = sEntry[0];
            // Get entry in target
            const tEntry = target[key];
            // Set if it's not present in the target
            if (tEntry === undefined || tEntry === null)
                target[key] = sEntry[1];
        });
        return target;
    }
    ObjectUtils.populate = populate;
    function swap(left, right) {
        return {
            left: right,
            right: left
        };
    }
    ObjectUtils.swap = swap;
    function exists(obj) {
        return obj !== undefined && obj !== null;
    }
    ObjectUtils.exists = exists;
})(ObjectUtils = exports.ObjectUtils || (exports.ObjectUtils = {}));
//# sourceMappingURL=index.js.map